@@ -31,7 +31,7 @@ impl Markdown {
     if !color {
       return MadSkin::no_style();
     }
-    let is_light = terminal_light::luma().is_ok_and(|l| l > 0.6);
+    let is_light = is_light_terminal();
     if is_light {
       MadSkin::default_light()
     } else {
@@ -39,3 +39,14 @@ impl Markdown {
     }
   }
 }
+
+fn is_light_terminal() -> bool {
+  use terminal_light as tl;
+  // prefer using env instead of escape sequences
+  // https://github.com/ast-grep/ast-grep/issues/2114
+  if let Ok(color) = tl::env::bg_color() {
+    tl::Color::Ansi(color).luma() > 0.6
+  } else {
+    tl::luma().is_ok_and(|l| l > 0.6)
+  }
+}
