@@ -32,6 +32,7 @@ pub enum ErrorContext {
   WalkRuleDir(PathBuf),
   ReadRule(PathBuf),
   ParseRule(PathBuf),
+  InvalidRuleId(PathBuf),
   ParseTest(PathBuf),
   InvalidGlobalUtils,
   GlobPattern,
@@ -45,6 +46,7 @@ pub enum ErrorContext {
   StdInIsNotInteractive,
   PatternHasError,
   // Scan
+  DuplicateRuleId(String),
   DiagnosticError(usize),
   RuleNotSpecified,
   RuleNotFound(String),
@@ -82,7 +84,7 @@ impl ErrorContext {
       ReadConfiguration | ReadRule(_) | WalkRuleDir(_) | WriteFile(_) => 6,
       StdInIsNotInteractive => 7,
       ParseTest(_) | ParseRule(_) | ParseConfiguration | ParsePattern | InvalidGlobalUtils
-      | LangInjection => 8,
+      | LangInjection | DuplicateRuleId(_) | InvalidRuleId(_) => 8,
       GlobPattern | BuildGlobs => 9,
       CannotInferShell => 10,
       ProjectAlreadyExist | FileAlreadyExist(_) => 17,
@@ -155,6 +157,11 @@ impl ErrorMessage {
         "The file is not a valid ast-grep rule. Please refer to doc and fix the error.",
         CONFIG_GUIDE,
       ),
+      InvalidRuleId(file) => Self::new(
+        format!("Cannot infer rule id from {}", file.display()),
+        "The rule file name is not valid UTF-8. Please add an explicit `id` field to the rule.",
+        CONFIG_GUIDE,
+      ),
       GlobPattern => Self::new(
         "Cannot parse glob pattern in config",
         "The pattern in files/ignore is not a valid glob. Please refer to doc and fix the error.",
@@ -190,6 +197,11 @@ impl ErrorMessage {
         "The file is not a valid ast-grep test case. Please refer to doc and fix the error.",
         TEST_GUIDE,
       ),
+      DuplicateRuleId(id) => Self::new(
+        format!("Duplicate rule id `{id}` is found"),
+        "Multiple rule files have the same id. Please add a unique `id` field to each rule.",
+        CONFIG_GUIDE,
+      ),
       DiagnosticError(num) => Self::new(
         format!("{num} error(s) found in code."),
         "Scan succeeded and found error level diagnostics in the codebase.",
