@@ -3,7 +3,10 @@ use napi::bindgen_prelude::*;
 use napi_derive::napi;
 
 use super::NapiConfig;
-use crate::doc::{JsDoc, Wrapper};
+use crate::{
+  doc::{JsDoc, Wrapper},
+  napi_lang::NapiLang,
+};
 use ast_grep_core::source::Content;
 
 #[napi(object)]
@@ -100,7 +103,7 @@ impl SgNode {
   pub fn matches(&self, m: Either3<String, u16, NapiConfig>) -> Result<bool> {
     let lang = *self.inner.lang();
     match m {
-      Either3::A(pattern) => Ok(self.inner.matches(Pattern::new(&pattern, lang))),
+      Either3::A(pattern) => Ok(self.inner.matches(napi_pattern(&pattern, lang)?)),
       Either3::B(kind) => Ok(self.inner.matches(KindMatcher::from_id(kind))),
       Either3::C(config) => {
         let pattern = config.parse_with(lang)?;
@@ -113,7 +116,7 @@ impl SgNode {
   pub fn inside(&self, m: Either3<String, u16, NapiConfig>) -> Result<bool> {
     let lang = *self.inner.lang();
     match m {
-      Either3::A(pattern) => Ok(self.inner.inside(Pattern::new(&pattern, lang))),
+      Either3::A(pattern) => Ok(self.inner.inside(napi_pattern(&pattern, lang)?)),
       Either3::B(kind) => Ok(self.inner.inside(KindMatcher::from_id(kind))),
       Either3::C(config) => {
         let pattern = config.parse_with(lang)?;
@@ -126,7 +129,7 @@ impl SgNode {
   pub fn has(&self, m: Either3<String, u16, NapiConfig>) -> Result<bool> {
     let lang = *self.inner.lang();
     match m {
-      Either3::A(pattern) => Ok(self.inner.has(Pattern::new(&pattern, lang))),
+      Either3::A(pattern) => Ok(self.inner.has(napi_pattern(&pattern, lang)?)),
       Either3::B(kind) => Ok(self.inner.has(KindMatcher::from_id(kind))),
       Either3::C(config) => {
         let pattern = config.parse_with(lang)?;
@@ -139,7 +142,7 @@ impl SgNode {
   pub fn precedes(&self, m: Either3<String, u16, NapiConfig>) -> Result<bool> {
     let lang = *self.inner.lang();
     match m {
-      Either3::A(pattern) => Ok(self.inner.precedes(Pattern::new(&pattern, lang))),
+      Either3::A(pattern) => Ok(self.inner.precedes(napi_pattern(&pattern, lang)?)),
       Either3::B(kind) => Ok(self.inner.precedes(KindMatcher::from_id(kind))),
       Either3::C(config) => {
         let pattern = config.parse_with(lang)?;
@@ -149,13 +152,13 @@ impl SgNode {
   }
 
   #[napi]
-  pub fn follows(&self, m: Either3<String, u16, NapiConfig>) -> bool {
+  pub fn follows(&self, m: Either3<String, u16, NapiConfig>) -> Result<bool> {
     let lang = *self.inner.lang();
-    match m {
-      Either3::A(pattern) => self.inner.follows(Pattern::new(&pattern, lang)),
+    Ok(match m {
+      Either3::A(pattern) => self.inner.follows(napi_pattern(&pattern, lang)?),
       Either3::B(kind) => self.inner.follows(KindMatcher::from_id(kind)),
       Either3::C(config) => self.inner.follows(config.parse_with(lang).unwrap()),
-    }
+    })
   }
 
   #[napi]
@@ -226,7 +229,7 @@ impl SgNode {
     let lang = *reference.inner.lang();
     let node_match = match matcher {
       Either3::A(pattern) => {
-        let pattern = Pattern::new(&pattern, lang);
+        let pattern = napi_pattern(&pattern, lang)?;
         reference.inner.find(pattern)
       }
       Either3::B(kind) => {
@@ -266,7 +269,7 @@ impl SgNode {
     let lang = *reference.inner.lang();
     let all_matches: Vec<_> = match matcher {
       Either3::A(pattern) => {
-        let pattern = Pattern::new(&pattern, lang);
+        let pattern = napi_pattern(&pattern, lang)?;
         reference.inner.find_all(pattern).collect()
       }
       Either3::B(kind) => {
@@ -429,3 +432,8 @@ impl SgRoot {
     Ok(self.1.clone())
   }
 }
+
+fn napi_pattern(pattern: &str, lang: NapiLang) -> Result<Pattern> {
+  Pattern::try_new(pattern, lang)
+    .map_err(|e| napi::Error::new(napi::Status::InvalidArg, e.to_string()))
+}
